==> grlib-linux-drvpkg-1.0.0.tar.bz2: grlib-linux-drvpkg-1.0.0/user/grspw_router/spwrouter_custom_config.c <==
/*
 * Custom configuration of SpaceWire Router demonstration software
 *
 * Copyright (c) 2010 Aeroflex Gaisler AB
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>

#include "grspw_router_lib.h"

void *router;
struct router_hw_info router_hw;
int router_nports;

struct router_config router_cfg =
{
	.flags = ROUTER_FLG_CFG | ROUTER_FLG_IID /*| ROUTER_FLG_TPRES | ROUTER_FLG_TRLD*/,
	.config = 0x10, /* Self-Address Enable */
	.iid = 1,
/*	.timer_prescaler = 0x123,
	.timer_reload = {
	}*/
};

struct router_routes routes =
{
	{
		/* 020..027 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 020..02f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 030..037 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 030..03f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 040..047 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 040..04f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 050..057 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 050..05f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 060..067 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 060..06f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 070..077 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 070..07f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 080..087 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 080..08f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 090..097 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 090..09f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 0a0..0a7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0a0..0af */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0b0..0b7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0b0..0bf */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 0c0..0c7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0c0..0cf */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0d0..0d7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0d0..0df */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 0e0..0e7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0e8..0ef */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0f0..0f7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0f8..0ff */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
	}
};

struct router_ps ps =
{
	.ps = /* Loopback... */
	{
		            0x00000002, 0x00000004, 0x00000008,
		0x00000010, 0x00000020, 0x00000040, 0x00000080,
		0x00000100, 0x00000200, 0x00000400, 0x00000800,
		0x00001000, 0x00002000, 0x00004000, 0x00008000,
		0x00010000, 0x00020000, 0x00040000, 0x00080000,
		0x00100000, 0x00200000, 0x00400000, 0x00800000,
		0x01000000, 0x02000000, 0x04000000, 0x08000000,
		0x10000000, 0x20000000, 0x40000000, 0x80000000,
	},
	.ps_logical =
	{
		/* 020..027 */ 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002,
		/* 028..02f */ 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002,
		/* 030..037 */ 0x00000004, 0x00000004, 0x00000004, 0x00000004, 0x00000004, 0x00000004, 0x00000004, 0x00000004,
		/* 038..03f */ 0x00000008, 0x00000008, 0x00000008, 0x00000008, 0x00000008, 0x00000008, 0x00000008, 0x00000008,

		/* 040..047 */ 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010,
		/* 048..04f */ 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020,
		/* 050..057 */ 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040,
		/* 058..05f */ 0x00000080, 0x00000080, 0x00000080, 0x00000080, 0x00000080, 0x00000080, 0x00000080, 0x00000080,

		/* 060..067 */ 0x00000100, 0x00000100, 0x00000100, 0x00000100, 0x00000100, 0x00000100, 0x00000100, 0x00000100,
		/* 068..06f */ 0x00000200, 0x00000200, 0x00000200, 0x00000200, 0x00000200, 0x00000200, 0x00000200, 0x00000200,
		/* 070..077 */ 0x00000400, 0x00000400, 0x00000400, 0x00000400, 0x00000400, 0x00000400, 0x00000400, 0x00000400,
		/* 078..07f */ 0x00000800, 0x00000800, 0x00000800, 0x00000800, 0x00000800, 0x00000800, 0x00000800, 0x00000800,

		/* 080..087 */ 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000,
		/* 088..08f */ 0x00002000, 0x00002000, 0x00002000, 0x00002000, 0x00002000, 0x00002000, 0x00002000, 0x00002000,
		/* 090..097 */ 0x00004000, 0x00004000, 0x00004000, 0x00004000, 0x00004000, 0x00004000, 0x00004000, 0x00004000,
		/* 098..09f */ 0x00008000, 0x00008000, 0x00008000, 0x00008000, 0x00008000, 0x00008000, 0x00008000, 0x00008000,

		/* 0a0..0a7 */ 0x00010000, 0x00010000, 0x00010000, 0x00010000, 0x00010000, 0x00010000, 0x00010000, 0x00010000,
		/* 0a8..0af */ 0x00020000, 0x00020000, 0x00020000, 0x00020000, 0x00020000, 0x00020000, 0x00020000, 0x00020000,
		/* 0b0..0b7 */ 0x00040000, 0x00040000, 0x00040000, 0x00040000, 0x00040000, 0x00040000, 0x00040000, 0x00040000,
		/* 0b8..0bf */ 0x00080000, 0x00080000, 0x00080000, 0x00080000, 0x00080000, 0x00080000, 0x00080000, 0x00080000,

		/* 0c0..0c7 */ 0x00100000, 0x00100000, 0x00100000, 0x00100000, 0x00100000, 0x00100000, 0x00100000, 0x00100000,
		/* 0c8..0cf */ 0x00200000, 0x00200000, 0x00200000, 0x00200000, 0x00200000, 0x00200000, 0x00200000, 0x00200000,
		/* 0d0..0d7 */ 0x00400000, 0x00400000, 0x00400000, 0x00400000, 0x00400000, 0x00400000, 0x00400000, 0x00400000,
		/* 0d8..0df */ 0x00800000, 0x00800000, 0x00800000, 0x00800000, 0x00800000, 0x00800000, 0x00800000, 0x00800000,

		/* 0e0..0e7 */ 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000,
		/* 0e8..0ef */ 0x02000000, 0x02000000, 0x02000000, 0x02000000, 0x02000000, 0x02000000, 0x02000000, 0x02000000,
		/* 0f0..0f7 */ 0x04000000, 0x04000000, 0x04000000, 0x04000000, 0x04000000, 0x04000000, 0x04000000, 0x04000000,
		/* 0f8..0ff */ 0x08000000, 0x08000000, 0x10000000, 0x10000000, 0x20000000, 0x20000000, 0x40000000, 0x80000000,
	},
};

char *linkstate_names[7] =
{
	/* 000 */ "Error Reset",
	/* 001 */ "Error Wait",
	/* 010 */ "Ready",
	/* 011 */ "Started",
	/* 100 */ "Connecting",
	/* 101 */ "Run",
	"N/A",
};

char *type_names[4] =
{
	"CFG ",
	"SpW ",
	"AMBA",
	"FIFO"
};

enum port_type {
	PT_CFG = 0,
	PT_SPW = 1,
	PT_AMBA = 2,
	PT_FIFO = 3,
};

int router_configure(void)
{
	unsigned int ctrl, sts, cfgsts;
	int i, linkstate, type;
	char *typestr;

	printf("Configuring Router\n");

	router = router_init("/dev/spwrouter0");
	if ( router == NULL ){
		printf("Failed to open SpW Router\n");
		return -1;
	}

	if ( router_hwinfo(router, &router_hw) ) {
		return -2;
	}
	router_print_hwinfo(&router_hw);

	if ( router_set_we(router, 0) ) {
		return -3;
	}

	if ( router_set_config(router, &router_cfg, &ps, &routes) ) {
		return -4;
	}

	if ( router_get_cfgsts(router, &cfgsts) ) {
		return -5;
	}
	printf("ROUTER CFG/STS: 0x%08x\n", cfgsts);

	router_nports = router_hw.nports_spw + router_hw.nports_amba +
			router_hw.nports_fifo;
	/* Print Current Link/Port Control/Status */
	for (i=0; i<router_nports+1; i++) {
		if ( router_port_get(router, i, &ctrl, &sts) ) {
			return -6;
		}

		if ( i == 0 ) {
			type = PT_CFG;
		} else if ( i < (1+router_hw.nports_spw) ) {
			type = PT_SPW;
		} else if ( i < (1+router_hw.nports_spw+router_hw.nports_amba) ) {
			type = PT_AMBA;
		} else {
			type = PT_FIFO;
		}
		typestr = type_names[type];

		if ( type != PT_SPW )
			linkstate = 6; /* N/A */
		else
			linkstate = (sts >> 12) & 0x7;

		printf("PORT[%02d]:  TYPE=%s CTRL=0x%08x  STATUS=0x%08x  LINKSTATE=%s\n",
			i, typestr, ctrl, sts, linkstate_names[linkstate]);
	}

	/* Enable and Start all Links */
	printf("Activating all Links/Ports\n");
	for (i=0; i<router_nports+1; i++) {

		if ( i == 0 ) {
			type = PT_CFG;
		} else if ( i < (1+router_hw.nports_spw) ) {
			type = PT_SPW;
		} else if ( i < (1+router_hw.nports_spw+router_hw.nports_amba) ) {
			type = PT_AMBA;
		} else {
			type = PT_FIFO;
		}

		switch (type) {
		default:
		case PT_CFG:
			break;
		case PT_SPW: /* SpaceWire Port */
			if ( router_port_ctrl(router, i, 0x0100002e) ) {
				return -10;
			}
			break;
		case PT_AMBA: /* AMBA Port */
			if ( router_port_ctrl(router, i, 0x00000028) ) {
				return -11;
			}
			break;
		case PT_FIFO: /* FIFO Port */
			if ( router_port_ctrl(router, i, 0x00000028) ) {
				return -12;
			}
			break;
		}
	}

	/* Print Current Link Status */
	for (i=1; i<router_hw.nports_spw+1; i++) {
		if ( router_port_get(router, i, &ctrl, &sts) ) {
			return -6;
		}

		if ( i == 0 ) {
			type = PT_CFG;
		} else if ( i < (1+router_hw.nports_spw) ) {
			type = PT_SPW;
		} else if ( i < (1+router_hw.nports_spw+router_hw.nports_amba) ) {
			type = PT_AMBA;
		} else {
			type = PT_FIFO;
		}
		typestr = type_names[type];

		if ( type != PT_SPW )
			linkstate = 6; /* N/A */
		else
			linkstate = (sts >> 12) & 0x7;

		printf("PORT[%02d]:  TYPE=%s   CTRL=0x%08x   STATUS=0x%08x   LINK-STATE=%s\n",
			i, typestr, ctrl, sts, linkstate_names[linkstate]);
	}

	return 0;
}

int main(int argc, char *argv[])
{
	int result;

	result = router_configure();
	if ( result ) {
		printf("Router_configure: Error %d\n", result);
		return result;
	}

	return 0;
}

==> grlib-linux-drvpkg-1.1.1.tar.bz2: grlib-linux-drvpkg-1.1.1/user/grspw_router/spwrouter_custom_config.c <==
/*
 * Custom configuration of SpaceWire Router demonstration software
 *
 * Copyright (c) 2016 Cobham Gaisler AB
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 * MA 02110-1301 USA
 */

#include <stdio.h>
#include <stdlib.h>

#include "grspw_router_lib.h"

void *router;
struct router_hw_info router_hw;
int router_nports;

struct router_config router_cfg =
{
	.flags = ROUTER_FLG_CFG | ROUTER_FLG_IID /*| ROUTER_FLG_TPRES | ROUTER_FLG_TRLD*/,
	.config = 0x10, /* Self-Address Enable */
	.iid = 1,
/*	.timer_prescaler = 0x123,
	.timer_reload = {
	}*/
};

struct router_routes routes =
{
	{
		/* Routing table entries for logical addresses 32-255
		 * Entries enabled, with low priority and no header deletion.
		 */
		/* 020..027 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 020..02f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 030..037 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 030..03f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 040..047 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 040..04f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 050..057 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 050..05f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 060..067 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 060..06f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 070..077 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 070..07f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 080..087 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 080..08f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 090..097 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 090..09f */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 0a0..0a7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0a0..0af */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0b0..0b7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0b0..0bf */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 0c0..0c7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0c0..0cf */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0d0..0d7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0d0..0df */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,

		/* 0e0..0e7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0e8..0ef */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0f0..0f7 */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
		/* 0f8..0ff */ 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5, 0x5,
	}
};

struct router_ps ps =
{
	/* Port setup for ports 1-31. */
	/* Loopback packets on incoming port. No packet distribution. */
	.ps =
	{
		            0x00000002, 0x00000004, 0x00000008,
		0x00000010, 0x00000020, 0x00000040, 0x00000080,
		0x00000100, 0x00000200, 0x00000400, 0x00000800,
		0x00001000, 0x00002000, 0x00004000, 0x00008000,
		0x00010000, 0x00020000, 0x00040000, 0x00080000,
		0x00100000, 0x00200000, 0x00400000, 0x00800000,
		0x01000000, 0x02000000, 0x04000000, 0x08000000,
		0x10000000, 0x20000000, 0x40000000, 0x80000000,
	},
	/* Port setup for logical addresses 32-255 */
	.ps_logical =
	{
		/* 020..027 -> port 1 */
		/* 020..027 */ 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002,
		/* 028..02f -> port 1 */
		/* 028..02f */ 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002, 0x00000002,
		/* 030..037 -> port 2 */
		/* 030..037 */ 0x00000004, 0x00000004, 0x00000004, 0x00000004, 0x00000004, 0x00000004, 0x00000004, 0x00000004,
		/* 038..03f -> port 3, etc... */
		/* 038..03f */ 0x00000008, 0x00000008, 0x00000008, 0x00000008, 0x00000008, 0x00000008, 0x00000008, 0x00000008,

		/* 040..047 */ 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010, 0x00000010,
		/* 048..04f */ 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020, 0x00000020,
		/* 050..057 */ 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040, 0x00000040,
		/* 058..05f */ 0x00000080, 0x00000080, 0x00000080, 0x00000080, 0x00000080, 0x00000080, 0x00000080, 0x00000080,

		/* 060..067 */ 0x00000100, 0x00000100, 0x00000100, 0x00000100, 0x00000100, 0x00000100, 0x00000100, 0x00000100,
		/* 068..06f */ 0x00000200, 0x00000200, 0x00000200, 0x00000200, 0x00000200, 0x00000200, 0x00000200, 0x00000200,
		/* 070..077 */ 0x00000400, 0x00000400, 0x00000400, 0x00000400, 0x00000400, 0x00000400, 0x00000400, 0x00000400,
		/* 078..07f */ 0x00000800, 0x00000800, 0x00000800, 0x00000800, 0x00000800, 0x00000800, 0x00000800, 0x00000800,

		/* 080..087 */ 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000, 0x00001000,
		/* 088..08f */ 0x00002000, 0x00002000, 0x00002000, 0x00002000, 0x00002000, 0x00002000, 0x00002000, 0x00002000,
		/* 090..097 */ 0x00004000, 0x00004000, 0x00004000, 0x00004000, 0x00004000, 0x00004000, 0x00004000, 0x00004000,
		/* 098..09f */ 0x00008000, 0x00008000, 0x00008000, 0x00008000, 0x00008000, 0x00008000, 0x00008000, 0x00008000,

		/* 0a0..0a7 */ 0x00010000, 0x00010000, 0x00010000, 0x00010000, 0x00010000, 0x00010000, 0x00010000, 0x00010000,
		/* 0a8..0af */ 0x00020000, 0x00020000, 0x00020000, 0x00020000, 0x00020000, 0x00020000, 0x00020000, 0x00020000,
		/* 0b0..0b7 */ 0x00040000, 0x00040000, 0x00040000, 0x00040000, 0x00040000, 0x00040000, 0x00040000, 0x00040000,
		/* 0b8..0bf */ 0x00080000, 0x00080000, 0x00080000, 0x00080000, 0x00080000, 0x00080000, 0x00080000, 0x00080000,

		/* 0c0..0c7 */ 0x00100000, 0x00100000, 0x00100000, 0x00100000, 0x00100000, 0x00100000, 0x00100000, 0x00100000,
		/* 0c8..0cf */ 0x00200000, 0x00200000, 0x00200000, 0x00200000, 0x00200000, 0x00200000, 0x00200000, 0x00200000,
		/* 0d0..0d7 */ 0x00400000, 0x00400000, 0x00400000, 0x00400000, 0x00400000, 0x00400000, 0x00400000, 0x00400000,
		/* 0d8..0df */ 0x00800000, 0x00800000, 0x00800000, 0x00800000, 0x00800000, 0x00800000, 0x00800000, 0x00800000,

		/* 0e0..0e7 */ 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000, 0x01000000,
		/* 0e8..0ef */ 0x02000000, 0x02000000, 0x02000000, 0x02000000, 0x02000000, 0x02000000, 0x02000000, 0x02000000,
		/* 0f0..0f7 */ 0x04000000, 0x04000000, 0x04000000, 0x04000000, 0x04000000, 0x04000000, 0x04000000, 0x04000000,
		/* 0f8..0ff */ 0x08000000, 0x08000000, 0x10000000, 0x10000000, 0x20000000, 0x20000000, 0x40000000, 0x80000000,
	},
};

char *linkstate_names[7] =
{
	/* 000 */ "Error Reset",
	/* 001 */ "Error Wait",
	/* 010 */ "Ready",
	/* 011 */ "Started",
	/* 100 */ "Connecting",
	/* 101 */ "Run",
	"N/A",
};

char *type_names[4] =
{
	"CFG ",
	"SpW ",
	"AMBA",
	"FIFO"
};

enum port_type {
	PT_CFG = 0,
	PT_SPW = 1,
	PT_AMBA = 2,
	PT_FIFO = 3,
};

int router_configure(void)
{
	unsigned int ctrl, sts, cfgsts;
	int i, linkstate, type;
	char *typestr;

	printf("Configuring Router\n");

	router = router_init("/dev/spwrouter0");
	if ( router == NULL ){
		printf("Failed to open SpW Router\n");
		return -1;
	}

	if ( router_hwinfo(router, &router_hw) ) {
		return -2;
	}
	router_print_hwinfo(&router_hw);

	if ( router_set_we(router, 0) ) {
		return -3;
	}

	if ( router_set_config(router, &router_cfg, &ps, &routes) ) {
		return -4;
	}

	if ( router_get_cfgsts(router, &cfgsts) ) {
		return -5;
	}
	printf("ROUTER CFG/STS: 0x%08x\n", cfgsts);

	router_nports = router_hw.nports_spw + router_hw.nports_amba +
			router_hw.nports_fifo;
	/* Print Current Link/Port Control/Status */
	for (i=0; i<router_nports+1; i++) {
		if ( router_port_get(router, i, &ctrl, &sts) ) {
			return -6;
		}

		if ( i == 0 ) {
			type = PT_CFG;
		} else if ( i < (1+router_hw.nports_spw) ) {
			type = PT_SPW;
		} else if ( i < (1+router_hw.nports_spw+router_hw.nports_amba) ) {
			type = PT_AMBA;
		} else {
			type = PT_FIFO;
		}
		typestr = type_names[type];

		if ( type != PT_SPW )
			linkstate = 6; /* N/A */
		else
			linkstate = (sts >> 12) & 0x7;

		printf("PORT[%02d]:  TYPE=%s  CTRL=0x%08x  STATUS=0x%08x  LINKSTATE=%s\n",
			i, typestr, ctrl, sts, linkstate_names[linkstate]);
	}

	/* Enable and Start all Links */
	printf("Activating all Links/Ports\n");
	for (i=0; i<router_nports+1; i++) {

		if ( i == 0 ) {
			type = PT_CFG;
		} else if ( i < (1+router_hw.nports_spw) ) {
			type = PT_SPW;
		} else if ( i < (1+router_hw.nports_spw+router_hw.nports_amba) ) {
			type = PT_AMBA;
		} else {
			type = PT_FIFO;
		}

		switch (type) {
		default:
		case PT_CFG:
			break;
		case PT_SPW: /* SpaceWire Port */
			if ( router_port_ctrl(router, i, 0x0100002e) ) {
				return -10;
			}
			break;
		case PT_AMBA: /* AMBA Port */
			if ( router_port_ctrl(router, i, 0x00000028) ) {
				return -11;
			}
			break;
		case PT_FIFO: /* FIFO Port */
			if ( router_port_ctrl(router, i, 0x00000028) ) {
				return -12;
			}
			break;
		}
	}

	/* Print Current Link Status */
	for (i=1; i<router_hw.nports_spw+1; i++) {
		if ( router_port_get(router, i, &ctrl, &sts) ) {
			return -6;
		}

		if ( i == 0 ) {
			type = PT_CFG;
		} else if ( i < (1+router_hw.nports_spw) ) {
			type = PT_SPW;
		} else if ( i < (1+router_hw.nports_spw+router_hw.nports_amba) ) {
			type = PT_AMBA;
		} else {
			type = PT_FIFO;
		}
		typestr = type_names[type];

		if ( type != PT_SPW )
			linkstate = 6; /* N/A */
		else
			linkstate = (sts >> 12) & 0x7;

		printf("PORT[%02d]:  TYPE=%s CTRL=0x%08x  STATUS=0x%08x  LINKSTATE=%s\n",
			i, typestr, ctrl, sts, linkstate_names[linkstate]);
	}

	return 0;
}

int main(int argc, char *argv[])
{
	int result;

	result = router_configure();
	if ( result ) {
		printf("Router_configure: Error %d\n", result);
		return result;
	}

	return 0;
}
